"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScriptService = void 0;
const tslib_1 = require("tslib");
const CP = tslib_1.__importStar(require("child_process"));
const FS = tslib_1.__importStar(require("fs"));
const Path = tslib_1.__importStar(require("path"));
const villa = tslib_1.__importStar(require("villa"));
const shared_1 = require("../shared");
const SCRIPTS_DIRECTORY_NAME = 'scripts';
const SCRIPTS_CONFIG_FILE_NAME_JSON = 'makescript.json';
class ScriptService {
    constructor(tiva, config) {
        this.tiva = tiva;
        this.config = config;
        this.ready = this.initialize();
    }
    get scriptsPath() {
        return Path.join(this.config.workspace, SCRIPTS_DIRECTORY_NAME);
    }
    get scriptsDefinitionPath() {
        return Path.join(this.scriptsPath, SCRIPTS_CONFIG_FILE_NAME_JSON);
    }
    get briefScriptDefinitions() {
        let scriptsDefinition = this.scriptsDefinition;
        if (!scriptsDefinition) {
            return [];
        }
        return scriptsDefinition.scripts.map(scriptDefinition => convertScriptDefinitionToBriefScriptDefinition(fillScriptDefinition(scriptDefinition, scriptsDefinition)));
    }
    async syncScripts() {
        shared_1.logger.info('Syncing scripts ...');
        try {
            if (FS.existsSync(this.scriptsPath)) {
                await villa.awaitable(CP.spawn('git', ['pull'], {
                    cwd: this.scriptsPath,
                }));
            }
            else {
                await villa.awaitable(CP.spawn('git', [
                    'clone',
                    this.config.scriptsRepoURL,
                    this.scriptsPath,
                ]));
            }
        }
        catch (error) {
            throw new Error(`Failed to sync scripts: ${error.message}`);
        }
        let scriptsDefinition = await this.parseScriptsDefinition();
        this.scriptsDefinition = scriptsDefinition;
        if (scriptsDefinition.initialize) {
            try {
                await villa.awaitable(CP.spawn(scriptsDefinition.initialize));
            }
            catch (error) {
                throw new Error(`Cannot to initial script repo with \`${scriptsDefinition.initialize}\`: ${error.message}`);
            }
        }
    }
    getScriptDefinitionByName(name) {
        let scriptsDefinition = this.scriptsDefinition;
        if (!scriptsDefinition) {
            return;
        }
        let definition = scriptsDefinition.scripts.find(definition => definition.name === name);
        if (!definition) {
            return undefined;
        }
        return fillScriptDefinition(definition, scriptsDefinition);
    }
    resolveSource(script) {
        return Path.join(this.scriptsPath, script.source);
    }
    async initialize() {
        await this.syncScripts();
    }
    async parseScriptsDefinition() {
        if (!FS.existsSync(this.scriptsPath)) {
            throw new Error(`Scripts repo not cloned`);
        }
        if (!FS.existsSync(this.scriptsDefinitionPath)) {
            throw new Error('Scripts definition not found: \n' +
                `please ensure the definition file "${SCRIPTS_CONFIG_FILE_NAME_JSON}" is existing in scripts repo.`);
        }
        shared_1.logger.info('Checking scripts definition file ...');
        let scriptsDefinitionBuffer = await villa.async(FS.readFile)(this.scriptsDefinitionPath);
        let scriptsDefinitionContent = scriptsDefinitionBuffer.toString();
        let parsedDefinition = JSON.parse(scriptsDefinitionContent);
        try {
            await this.tiva.validate({
                module: '@makeflow/makescript-agent',
                type: 'ScriptsDefinition',
            }, parsedDefinition);
        }
        catch (error) {
            if (error.diagnostics) {
                shared_1.logger.error(`The structure of the scripts definition file "${this.scriptsDefinitionPath}" not matched: \n` +
                    `    ${error.diagnostics}`);
                process.exit(1);
            }
            else {
                throw error;
            }
        }
        return parsedDefinition;
    }
}
exports.ScriptService = ScriptService;
function fillScriptDefinition(definition, scriptsDefinition) {
    var _a;
    return {
        ...definition,
        passwordHash: (_a = definition.passwordHash) !== null && _a !== void 0 ? _a : scriptsDefinition.passwordHash,
        hooks: {
            ...scriptsDefinition.hooks,
            ...definition.hooks,
        },
    };
}
function convertScriptDefinitionToBriefScriptDefinition(definition) {
    var _a, _b;
    return {
        displayName: definition.displayName,
        name: definition.name,
        type: definition.type,
        manual: !!definition.manual,
        parameters: (_a = definition.parameters) !== null && _a !== void 0 ? _a : [],
        needsPassword: !!definition.passwordHash,
        hooks: {
            postTrigger: !!((_b = definition.hooks) === null || _b === void 0 ? void 0 : _b.postTrigger),
        },
    };
}
//# sourceMappingURL=script-service.js.map